import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Random;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


public class MainCreator {
	private static int compNum;
	private static int procNum;
	private static int maxReplica;
	private static Random random;    // pseudo-random number generator
	private static long seed;        // pseudo-random number generator seed

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		seed = System.currentTimeMillis();
		random = new Random(seed);

		readXML();
		System.out.println(compNum + " " + procNum);
		createMain();
		// TODO Auto-generated method stub

	}


	private static void createMain()
	{
		try {
			String path = new java.io.File(".").getCanonicalPath();
			FileWriter outFile = new FileWriter( path + "\\src\\Main_" + compNum + "_Comps_" + procNum + "_Procs_"+ maxReplica + "_MaxRep.java");
			PrintWriter out = new PrintWriter(outFile);
			// Write text to file
			out.println("public class Main_" + compNum + "_Comps_" + procNum + "_Procs_"+ maxReplica + "_MaxRep {");
			out.println("public static void main(String[] args) {");
			out.println("try {");
			out.println("Architecture arch = new Architecture(" + compNum + ", " + procNum + ");");
			out.println("arch.setProcessMemReq(1);");			
			out.println("arch.setAvailableMem(200000);");
			out.println("arch.setProcessCPUReq(1);");
			out.println("arch.setAvailableCPU(100000);");
			for (int i = 0; i < compNum; i++)
			{
				out.println();
				out.println("arch.setComponentReliability(" + i + ", " + uniform(0.96, 1) + ");");
				out.println("arch.deployComponent(" + i + ", " + uniform(0, procNum) + ");");
				out.println("arch.setMaxReplicas(" + i + ", " + uniform(1, maxReplica) + ");");
				out.println("arch.setComponentMemUsage(" + i + ", 1);");
				out.println("arch.setComponentCPUUsage(" + i + ", 1);");

			}

			out.println("double[][] opProfile = new double[" + new Integer(compNum + 1) + "][" + new Integer(compNum + 1) + "];");

			
			// Change subSystem value to generate profiles with different number of subsytems 
			int subSystem = 20;
			int counter = 0;
			int ControllerIndex = 1;
			
			for (int i = 0; i < subSystem; i++)
			{
				for (int j = 0; j < (compNum / subSystem); j++)
				{
					if (counter != ControllerIndex)					
						out.println("opProfile[" + counter + "][" + ControllerIndex + "] = 1;");
					counter++;
				}							
				ControllerIndex += (compNum / subSystem);
			}
			out.println();
			
			
			double equal = 1.0 / (compNum / subSystem);
			double randomShare = 0;
			double total = 0;
			counter = 2;
			ControllerIndex = 1;
			for (int i = 1; i <= subSystem; i++)
			{
				for (int j = 2; j <= (compNum / subSystem); j++)
				{
					if (total < 1)
						if (j == (compNum / subSystem))
							randomShare = 1.0 - total;
						else
						{
							randomShare = uniform(-(equal/2.0), 1 * (equal/2.0)) + equal;
							total += randomShare;

						}
					else
						randomShare = 0;
						out.println("opProfile[" + ControllerIndex + "][" + counter + "] = " + randomShare + ";");					
					counter++;
				}
				counter++;
				total = 0;
				ControllerIndex += (compNum / subSystem);
			}
			out.println();
			
			

			
			// One controller and all others interact just with the controller
			
/*			 for (int i = 0; i < compNum; i++)
				if (i != 1)
					out.println("opProfile[" + i + "][1] = 1;");

			out.println(); 
			double equal = (double) 1 / compNum; 
			double randomShare = 0;
			double total = 0;

			for (int i = 1; i < compNum; i++)
			{
				if (total < 1)
					if (i == (compNum - 1))
						randomShare = 1.0 - total;
					else
					{
						randomShare = uniform(-(equal/2.0), 1 * (equal/2.0)) + equal;
						total += randomShare;

					}
				else
					randomShare = 0;	
				out.println("opProfile[1][" + new Integer(i + 1) + "] = " + randomShare + ";");

			}*/

			/*for (int i = 1; i < compNum; i++)
					out.println("opProfile[1][" + new Integer(i + 1) + "] = " + equal + ";");*/

			out.println("arch.setOperationalProfile(opProfile);");
			out.println("arch.show();");
			out.println("long start = System.currentTimeMillis();");
			out.println("arch.optimizeArchitecture_New(opProfile, 99);");
			out.println("long elapsedTimeMillis = System.currentTimeMillis() - start;");
			out.println("System.out.println(\"Elapsed time : \" + elapsedTimeMillis);");
			out.println("} catch (Exception ex) {");
			out.println("System.out.print(ex.toString());");
			out.println("}");
			out.println("}");
			out.println("}");




			out.close();
		} catch (IOException e){
			e.printStackTrace();
		}
	}

	public static double uniform() {
		return random.nextDouble();
	}

	public static double uniform(double a, double b) {
		return a + uniform() * (b-a);
	}

	/**
	 * Return an integer uniformly between 0 and N-1.
	 */
	public static int uniform(int N) {
		return random.nextInt(N);
	}

	/**
	 * Return int uniformly in [a, b).
	 */
	public static int uniform(int a, int b) {
		return a + uniform(b - a);
	}

	private static void readXML()
	{
		try {
			File file = new File("conf/input.xml");
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document doc = db.parse(file);
			doc.getDocumentElement().normalize();
			NodeList compList = doc.getElementsByTagName("Components");
			Element firstNameElement = (Element)compList.item(0);
			NodeList textFNList = firstNameElement.getChildNodes();
			compNum = Integer.parseInt(textFNList.item(0).getNodeValue());             

			compList = doc.getElementsByTagName("Processes");
			firstNameElement = (Element)compList.item(0);
			textFNList = firstNameElement.getChildNodes();
			procNum = Integer.parseInt(textFNList.item(0).getNodeValue());

			compList = doc.getElementsByTagName("MaxReplica");
			firstNameElement = (Element)compList.item(0);
			textFNList = firstNameElement.getChildNodes();
			maxReplica = Integer.parseInt(textFNList.item(0).getNodeValue());


		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
